function downloadUtil() {
	var self = this;
	/**
	 * Make and http call and save the result into a file.
	 *
	 * Parameters:
	 * url - url of the http call to be downloaded.
	 * localFilePath - The path of the local file where the content will be saved. The folder in which the file to be downloaded must exsists.
	 * callBack_DownloadOneFileFinished - The fucntion that is called once the download has finished
	 * 		Callback return object:
	 * 			{ status, path}
	 * 		status: integer - HTTP status for the call. 200 means successful.
	 * 		path: string - full path of the file just downloaded
	 */
	self.downloadOneFile = function(url, localFilepath, callBack_DownloadOneFileFinished) {

		var c = Titanium.Network.createHTTPClient();

		if (null != callBack_DownloadOneFileFinished) {
			c.onerror = function(e) {
				Ti.API.info('MyApp: Download failed: url= ' + url + ' Error=' + e.error);

				callBack_DownloadOneFileFinished({
					status : e.error,
					path : ''
				});
			};

			c.onload = function(e) {

				if (Titanium.Platform.name === 'android') {

					// On android HTTPClient will not save the file to disk. So have to hack around it
					Ti.API.info('MyApp: (Andoid) Downloaded this file from server:.' + localFilepath);
					var f = Titanium.Filesystem.getFile(localFilepath);
					f.write(c.responseData);
				}

				callBack_DownloadOneFileFinished({
					status : c.status,
					path : localFilepath
				});
			};
		}

		c.open('GET', url);

		if (null != localFilepath && Titanium.Platform.name !== 'android') {
			Ti.API.info('MyApp:  (iOS) Downloaded this file from server:.' + localFilepath);
			c.file = Titanium.Filesystem.getFile(localFilepath);
		}

		c.send();
	};

	/**
	 * Given an array of URLs make several http calls and save the results into different file.
	 *
	 * Parameters:
	 * downloadQueue - [{'filepath' : "", 'url': ""}]
	 * 			filepath - The path of the local file where the content will be saved. The folder in which the file to be downloaded must exsists.
	 * 			url - url of the http call to be downloaded.
	 * callBack_DownloadOneFileFinished - The fucntion that is called once each file download has finished
	 * 		Callback return object:
	 * 			{ status, path}
	 * 		status: integer - HTTP status for the call. 200 means successful.
	 * 		path: string - full path of the file just downloaded
	 * callBack_DownloadMultipleFileFinished - The function that is caled once all the files are downloaded. This function does not accept any parameters
	 *
	 */
	self.downloadMultiFile = function(downloadQueue, callBack_DownloadOneFileFinished, callBack_DownloadMultipleFileFinished) {

		var queueIndex = 0;

		var processQueue = function(download_result) {
			//once the download of one file is finished the downloadOneFile will call back the processQueue
			//which will move the index forward and download another file

			if ( typeof (download_result) !== 'undefined') {
				callBack_DownloadOneFileFinished(download_result);
			}

			if (queueIndex < downloadQueue.length) {

				self.downloadOneFile(downloadQueue[queueIndex].url, downloadQueue[queueIndex].filepath, processQueue);
				queueIndex++;

			} else {
				callBack_DownloadMultipleFileFinished();
			}

		};
		processQueue();
	};

	function downloadOverlay(experience, btn_experience) {

		var overlayNumber = 360;
		if (experience == 'march') {
			overlayNumber = 240;
		}

		// --- OVERLAY DIR
		hideLoading();
		var overlayDir = Ti.Filesystem.getFile(Ti.Filesystem.applicationDataDirectory, 'overlay');
		if (!overlayDir.exists()) {
			overlayDir.createDirectory();
		}

		// DRECTORY WHERE THE OVERLAY IMAGES WILL BE DOWNLOADED
		var experienceDir = Ti.Filesystem.getFile(Ti.Filesystem.applicationDataDirectory, 'overlay/' + experience);

		// CHECK TO SEE IF THE DIRECTORY ALREADY EXSISTS
		if (!experienceDir.exists()) {
			experienceDir.createDirectory();
			Ti.API.info(experience + ' Directory Created');
		}

		Ti.API.info('===' + experienceDir.getNativePath());

		var experienceFileCount = experienceDir.getDirectoryListing();
		if (experienceFileCount.length < (overlayNumber - 5)) {

			var download_size = 30;
			if (experience == 'march') {
				download_size = 115;
			} else if (experience == 'dance') {
				download_size = 35;
			}

			// DIALOG QUESTIONING FOR DOWNLOAD
			var dlg = Titanium.UI.createAlertDialog({
				message : 'This experience is not yet installed, would you like to download it?\nSize of download: ' + download_size + 'M',
				buttonNames : ['No', 'Yes']
			});
			dlg.addEventListener('click', function(ev) {
				if (ev.index == 0) {// clicked "NO"
					Ti.API.info('DONT DOWNLOAD, DO NOTHING');
				} else {
					// START DOWNLOADING
					// CREATES THE ARRAY WITH THE URLS TO DOWNLOAD
					var overlayQ = [];
					for (var i = 0; i < overlayNumber; i++) {
						path = experienceDir.getNativePath() + "epicvoyage" + i + ".png";
						overlayQ.push({
							'filepath' : path,
							'url' : "https://s3.amazonaws.com/AntarcticaApp/" + experience + "/epicvoyage" + i + ".png"
						});
						path = path.replace("file://localhost", "");
					};

					var dwnload_count = 0;
					showLoading("Downloading 0%");
					self.downloadMultiFile(overlayQ, function(e) {
						activityIndicator.setMessage("Downloading " + (Math.round(((dwnload_count++) / overlayNumber) * 100)) + "%");
					}, function(e) {
						downloadOverlay(experience, btn_experience);
					});

				}
			});
			dlg.show();
		} else {
			hideLoading();
			var path;
			var slides = [];
			for (var i = 0; i < overlayNumber; i++) {
				path = experienceDir.getNativePath() + "epicvoyage" + i + ".png";
				path = path.replace("file://localhost", "");
				slides.push(path);
			};

			var btn_children = btn_experience.getChildren();
			if (btn_children.length < 1) {
				btn_experience.add(getCheckmark());
			}
			instructions_window = new Instructions(experience, slides);
			instructions_window.open(anm_win_open);
		}

		Ti.API.info(experience + ' == ' + experienceFileCount.length);
	}

	function checkDownloadOverlay(experience, btn_experience) {

		var overlayNumber = 360;
		if (experience == 'march') {
			overlayNumber = 240;
		}

		// DRECTORY WHERE THE OVERLAY IMAGES WILL BE DOWNLOADED
		var experienceDir = Ti.Filesystem.getFile(Ti.Filesystem.applicationDataDirectory, 'overlay/' + experience);

		// CHECK TO SEE IF THE DIRECTORY ALREADY EXSISTS
		if (experienceDir.exists()) {
			var experienceFileCount = experienceDir.getDirectoryListing();
			if (experienceFileCount.length > (overlayNumber - 5)) {
				btn_experience.add(getCheckmark());
			}
		}
	}

	return self;

}

module.exports = downloadUtil;
