/**
 * This will create the special scrollableview that allows circular paging.  Also,
 * it takes regular parameters specified by Ti.UI.ScrollableView
 * Note: need at least 5 pages for this to work
 */
exports.createScrollableView = function(params) {
	if (params.views.length < 5) {
		Ti.API.error('You need at least 5 pages to use this scrollable view');
		return null;
	}

	var scrollableViewObj = new ScrollableViewObj(params);

	return scrollableViewObj.scrollableView;
};
// this will define the scrollableView object
function ScrollableViewObj(params) {
	var thisRef = this;

	var backgroundView = Ti.UI.createView({
		backgroundColor : 'green'
	});

	// add additional parameters here
	params.showPagingControl = false;
	params.clipViews = false;

	this.views = params.views;
	// store the views for now
	this.numViews = this.views.length;

	params.views = this.reorderViews(0);

	var scrollableView = Ti.UI.createScrollableView(params);

	var overlayImage;

	var endScroll = true;
	scrollableView.addEventListener('scrollEnd', function(evt) {
		Ti.API.info('==Current Page : ' + evt.currentPage);
		var newPage = -1;
		Ti.API.info('==New Page : ' + newPage);
		if (evt.currentPage < 3) {
			newPage = thisRef.views.length + evt.currentPage;
		} else if (evt.currentPage >= (3 + thisRef.views.length - 1)) {
			newPage = evt.currentPage + 6 - this.views.length;
		}

		if (newPage > -1 && endScroll) {
			thisRef.scrollableView.setCurrentPage(newPage);
			endScroll = false;
		} else {
			endScroll = true;
		}
		Ti.API.info('==New Page : ' + newPage);
	});

	scrollableView.currentPage = 3;
	// start at 3

	this.scrollableView = scrollableView;
}

ScrollableViewObj.prototype.reorderViews = function(visibleViewNum) {

	// reorder views for scrollableView to create circular-effect
	var reorderedViews = [];
	// only going to be up to 5 views here

	// first 3 will be the last 3 duplicated
	var curIndex = this.numViews - 3;
	reorderedViews.push(this.duplicateView(this.views[curIndex]));
	curIndex++;
	reorderedViews.push(this.duplicateView(this.views[curIndex]));
	curIndex++;
	reorderedViews.push(this.duplicateView(this.views[curIndex]));

	// now, put the views in order
	for (var i = 0; i < this.numViews; i++) {
		reorderedViews.push(this.views[i]);
	}

	// now duplicate the first 3
	curIndex = 0;
	reorderedViews.push(this.duplicateView(this.views[curIndex]));
	curIndex++;
	reorderedViews.push(this.duplicateView(this.views[curIndex]));
	curIndex++;
	reorderedViews.push(this.duplicateView(this.views[curIndex]));

	return reorderedViews;
};

ScrollableViewObj.prototype.duplicateView = function(view) {
	if (view.image) {// image view!
		newView = Ti.UI.createImageView({
			image : view.image
		});
	} else {
		newView = Ti.UI.createView({
			backgroundImage : view.backgroundImage
		});
	}

	if (view.width) {
		newView.width = view.width;
	}
	if (view.height) {
		newView.height = view.height;
	}
	if (view.left) {
		newView.left = view.left;
	}
	if (view.top) {
		newView.top = view.top;
	}
	if (view.right) {
		newView.right = view.right;
	}
	if (view.bottom) {
		newView.bottom = view.bottom;
	}

	return newView;
};
